// File   : NucModel.hh
// Nuclear Model class
// Authors: F. Salamida 
// 
// date: 05/07/2011


#ifndef _NucModel_hh_
#define _NucModel_hh_

#include <cstdio>
#include <list>
#include <vector>
#include <fstream>
#include <TRandom3.h>
#include <Particle.h>


const int nstep = 300;
static int fLosses;
static std::vector<int> fZ;
static std::vector<int> fA;
static int k_com;
static double Gam_com;
static double z_com;
static double s_com;

double derivsA0 (double &, double &);
double fdisi (double );
double fdisi_IR (double);
double losseA0pair(double &, double &, const int &);
double losseA0Pi(double &, double &, const int &);
void sigma_disi (int &, double &, double &, double &, double &);

enum eModelType {
   eNone,
   eOnlyProton,
   eSalomon,
 };


class NucModel {

 public:
  
  //Constructors
  NucModel(eModelType model, int LossType, TRandom3 * );
  ~NucModel();
   
  //getters
  int GetNElements()    {return fnelements;}
  int GetLossType()    {return fLosses;}

  //functions
  void Validate(Particle* );
  int TestPrimary(int );
  Particle* GetRandomParticle(double, double);
  std::vector<Particle> PropagateParticle(Particle* );
  Particle* PropagateProton(Particle*);
  std::vector<Particle> PropagateNucleus(Particle*);
  std::vector<Particle> GetLosses(Particle *);
  double GetProcess(int &, double &, double &, Particle & );
  double GetEdetN(Particle *);
  int GetJNuc(int);
  double losseAdisi(int &, double &, double &, int &, int & ); 
  double evolveA0 (int &, double &, double &, double &);
  double BRdisi(int , int );


private:  

  int binary(int );
  int fnelements;
  eModelType fType;
  TRandom3* fRandom;
  
  double fe_1;
  double fe_max;
  std::vector<double> feth_1;
  std::vector<double> feth_2;
  std::vector<double> fe0_1;
  std::vector<double> fcsi_1;
  std::vector<double> fe0_2;
  std::vector<double> fcsi_2;
  std::vector<double> fdelta_2;
  std::vector<double> fzita;
  
  
};



#endif  // _NucModel_hh_
