#include <Output.h>

#include <sstream>
#include <iostream>

#include "TTree.h"
#include "TBranch.h"
#include "TFile.h"
#include "TDirectory.h"
#include "TString.h"

#include <PhysicsFunctions.h>

using namespace std;


Output::Output(char *filename) 
{

  fout = new TFile(filename,"RECREATE");
  int comp = 1;
  fout->SetCompressionLevel(comp);

  TDirectory* const save = gDirectory;
  // Spectrum output tree                                                       
  fout->cd();

  fnuc = new TTree("nuc","Prop Nuclei"); 
  fnuc->Branch("evt",&fevt,"evt/I");
  fnuc->Branch("branch",&fbranch,"branch/I");
  fnuc->Branch("intmult",&fintmult,"intmult/I");
  fnuc->Branch("Acurr",&fAcurr,"Acurr/I"); 
  fnuc->Branch("Zecurr",&fZecurr,"Zecurr/I"); 
  fnuc->Branch("zOri",&fzOri,"zOri/D"); 
  fnuc->Branch("zEnd",&fzEnd,"zEnd/D"); 
  fnuc->Branch("EOri",&fEOri,"EOri/D"); 
  fnuc->Branch("EEnd",&fEEnd,"EEnd/D"); 
  fnuc->Branch("Dist",&fDist,"Dist/D"); 
  fnuc->Branch("bdis",&fbdis,"bdis/D"); 

  fev = new TTree("ev","Events"); 
  fev->Branch("timexev",&ftimexev,"timexev/D");
  fev->Branch("branxev",&fbranxev,"branxev/I");
  fev->Branch("seed",&fRandomSeed,"RandomSeed/i");
  
  save->cd();

}


Output::~Output() {

}


void Output::Stream(int& evt, Particle *mypart) {

  fevt = evt;
  fbranch = mypart->GetBranch();
  fintmult = mypart->GetIntMult();
  fAcurr = mypart->GetMassNum();
  fZecurr = mypart->GetCharge();
  fzOri = mypart->GetZprod();
  fzEnd = mypart->GetZint(); 
  fEOri = mypart->GetEprod(); 
  fEEnd = mypart->GetEint();
  fDist = GetR(fzEnd, fzOri);
  fbdis = mypart->GetBetaDis(); 
  fnuc->Fill();

}


void Output::Stream(double& timexev, int& branxev, unsigned long int& cseed) {

  fbranxev = branxev;
  ftimexev = timexev;
  fRandomSeed = cseed;
  fev->Fill();

}


void Output::Finish() {

  TDirectory* const save = gDirectory;

  fout->cd();
  fnuc->Write();
  fev->Write();
  fout->Close();

  save->cd();

}
