// File   : Particle.hh
// Particle class
// Authors: F. Salamida 
// 
// date: 05/07/2011


#ifndef _Particle_hh_
#define _Particle_hh_

#include <cstdio>
#include <list>
#include <vector>
#include <fstream>

#include "Constants.h"
 
enum eParticleType {
   eNull,
   eNucleus,
   eMuon,
   eElectron,
   eProton,
   eNeutron,
   ePhoton,
   eNeutrino
 };

class Particle {

 public:
  
  //Constructors
  Particle();
  Particle(int massnum, int charge, int branch, double Eprod, double zprod, eParticleType type);
  Particle(int massnum, int charge, int branch, double Eprod, double Zprod, double Eint, double Zint, eParticleType type);
  ~Particle();
 
  
  //setters
  void SetMassNum(int val)  { fmassnum = val; }
  void SetCharge(int val)   { fcharge = val; }
  void SetBranch(int val)   { fbranch = val; }
  void SetIntMult(int val)  { fIntMult = val; }
  void SetEprod(double val) { fEprod = val; }
  void SetZprod(double val) { fZprod= val;}
  void SetEint(double val)  { fEint = val; }
  void SetZint(double val)  { fZint = val; }
  void SetGprod(double val) { fEprod = val * mN * fmassnum; }
  void SetGint(double val)  { fEint  = val * mN * fmassnum; }
  void SetBetaDis(double val)  { fBetaDis = val; }  
  void SetType(eParticleType val)  { fType = val; }

 //getters
  int GetNeutrons() {return fmassnum-fcharge;}
  int GetMassNum()  { return fmassnum; }
  int GetCharge()   { return fcharge; }
  int GetBranch()   { return fbranch; }
  int GetIntMult()  { return fIntMult; }
  double GetEprod() { return fEprod; }
  double GetZprod() { return fZprod;}
  double GetEint()  { return fEint; }
  double GetZint()  { return fZint; }
  double GetGprod() { return fEprod/(mN * fmassnum); }
  double GetGint()  { return fEint/(mN * fmassnum); }
  double GetBetaDis() { return fBetaDis; }  
  eParticleType GetType() { return fType; }
  std::string GetName();

 private:  

 int fmassnum; 
 int fcharge;
 int fbranch;
 int fIntMult;
 double fEprod;
 double fZprod;
 double fEint; 
 double fZint;
 double fBetaDis;
 eParticleType fType;

};

#endif  // _Particle_hh_
