#ifndef _PhysicsFunctions_h_
#define _PhysicsFunctions_h_


#include <iomanip>
#include <cstdio>
#include <iostream>
#include <fstream>
#include <cstdlib>
#include <list>
#include <cmath>

#include <MathFunctions.h>
#include <Constants.h>
#include "lossesN.h"
#include "lossesPI.h"
#include <sigmaA_disi.h>

using namespace std;

// -------------------------------------------------------------------------

double derivsN (double &z, double &LnEg)
{
  double lbeta_pg = 0.;
  double lEg = LnEg * log10(exp(1.)) + log10 (1.+z); 
  double * y2a = new double[nE_p];
  double beta_pg = 0.;
  double dtdz = 1. / (H0 * (1.+z) * sqrt(om_m * pow(1+z,3) + om_l));
  double dLnEgdz = 1. / (1.+z); 
  if (lEg >= 17.) {
    spline(LogE,lbeta0,nE_p,1.e30,1.e30,y2a);
    lbeta_pg = splint(LogE,lbeta0,y2a,nE_p,lEg);
    beta_pg = pow(1+z,3) * pow(10, lbeta_pg);
    dLnEgdz += dtdz * beta_pg;
  }

  delete [] y2a;
  return dLnEgdz;
}


double evolveN (double z_in, double Gam_in, double z_out)
{
  // .............................................................................
  //  Proton evolution according to the energy losses of BGG 2002, the generation 
  //  energy of protons is determined for any intitial condition (z_in,Gam_in).
  // .............................................................................

  static const double eps=1.e-9;
  double h1 =1.e-4;
  double hmin = 0.;

  int nok, nbad;
  
  double E0 = Gam_in * mN;
  double LnEg0 = log(E0);
  double LnEg = odeint(LnEg0, z_in, z_out, eps, h1, hmin, nok, nbad, derivsN);
  double Gam_out = -999.;
  Gam_out = exp(LnEg)/mN;
  return Gam_out;
}

double rr(double z)
{
  double ddz = 1./(H0 * sqrt(om_m * pow(1+z,3) + om_l));
  return SpeedOfLight*m2cm*SecInY*ddz/Mpc2cm;
}


double GetR(double& zmin, double& z)
{	
  return qgaus(rr,zmin,z);
}



// -------------------------------------------------------------------------

double W (const double &eth, const double &e0, const double &d0)
{
     
  double pi = 3.1416;

  // here we corrected a typo in eq. (3) of the Stecker & Salamon paper

  double x1 = (e_1-e0)/(d0/sqrt(2.));     
  double x2 = (e0-eth)/(d0/sqrt(2.));     

  double s_erfx1 = erf(x1);
  double s_erfx2 = erf(x2);

  double Wval = d0 * sqrt(pi/8.) * (s_erfx1 + s_erfx2);

  return Wval;
}



#endif
